# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

"""Randomly sample rows of a csv file
"""

import logging
import time

from csvpandas import utils

log = logging.getLogger(__name__)


def build_parser(parser):
    # required inputs
    parser.add_argument(
        'n',
        type=float,
        help='number of rows to sample.  Can be a decimal fraction.')
    parser.add_argument(
        '--seed-in',
        type=utils.opener('r'),
        help=('file containing integer to generate random seed'))
    parser.add_argument(
        '--seed-out',
        type=utils.opener('w'),
        help=('file containing integer used to generate seed'))

    parser.add_argument(
        '--rest',
        help='file to output rows not included in sample.')

    parser.add_argument(
        '--replace',
        action='store_true',
        help=('Sample with or without replacement.'))


def action(args):
    if args.seed_in:
        seed = int(args.seed_in.read().strip())
    else:
        seed = int(time.time())

    df = args.csv

    if args.n < 1:
        sample = df.sample(
            frac=args.n, replace=args.replace, random_state=seed)
    else:
        sample = df.sample(
            n=int(args.n), replace=args.replace, random_state=seed)

    sample.to_csv(args.out)

    if args.rest:
        df[~df.index.isin(sample.index)].to_csv(args.rest)

    if args.seed_out:
        args.seed_out.write(str(seed))



# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

"""Merge any number of csvpandas file column wide
"""

import logging
import pandas

log = logging.getLogger(__name__)


def build_parser(parser):
    # required inputs
    parser.add_argument(
        '--other',
        required=True,
        action='append',
        help='tabulated input file')

    parser.add_argument(
        '--how',
        choices=['inner', 'outer', 'left', 'right'],
        default='inner',
        help=('how to join %(choices)s'))
    parser.add_argument(
        '--on',
        help=('column delimite list of columns or single common column name'))
    parser.add_argument(
        '--comment', metavar='CHAR',
        help=('comment character'))


def action(args):
    others = []
    for other in args.other:
        others.append(pandas.read_csv(
            other,
            dtype=str,
            comment=args.comment,
            na_filter=False,
            sep=args.sep,
            header=None if args.no_header else 0))

    if args.on:
        on = args.on.split(',')

        if args.no_header:
            on = map(int, on)
    else:
        on = None

    for other in others:
        args.csv = args.csv.merge(other, how=args.how, on=on)

    args.csv.to_csv(args.out, header=not args.no_header, index=False)



# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

"""Output csv file in a command line interface friendly way.

FIXME: This might be better off as an extension of
pandas.core.format.TableFormatter.  Going from a DataFrame to string using
apply is a bit slow.
"""

import logging

log = logging.getLogger(__name__)


def build_parser(parser):
    return parser


def action(args):
    df = args.csv

    if df.empty:
        return

    header = df.columns

    # remove this 'Unnamed: #' default columns header if it exists
    header = list(map(
        lambda x: '' if x.startswith('Unnamed: ') else x, header))

    df.columns = ['column{}'.format(col) for col in xrange(len(df.columns))]
    header_dict = {col: header[i] for i, col in enumerate(df.columns)}

    # calculate column widths
    widths = {col: df[col].map(len).max() for col in df.columns}
    # check if header is longer
    for col, width in widths.items():
        widths[col] = max(len(str(header_dict[col])), width)

    # divider - add two spaces
    divider = '+'.join('-' * (widths[col] + 2) for col in df.columns)
    divider = '|' + divider + '|\n'

    # create column formats
    cols = ['{' + col + ':<' + str(widths[col]) + '}' for col in df.columns]
    # join columns into a row
    row = '| ' + ' | '.join(cols) + ' |\n'

    # write
    args.out.write(divider)

    if not args.no_header:
        args.out.write(row.format(**header_dict))
        args.out.write(divider)

    df.apply(lambda x: args.out.write(row.format(**x)), axis=1)

    args.out.write(divider)
